import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _defineProperty from "@babel/runtime/helpers/defineProperty";
const _excluded = ["max", "separator", "renderMore", "IconMore", "Component", "MoreProps", "className", "children"];
function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }
import React from 'react';
import { clamp, is, unique } from '@amaui/utils';
import { classNames, style as styleMethod, useAmauiTheme } from '@amaui/style-react';
import IconMaterialNavigateNext from '@amaui/icons-material-rounded-react/IconMaterialNavigateNextW100';
import IconMaterialMoreHoriz from '@amaui/icons-material-rounded-react/IconMaterialMoreHorizW100';
import LineElement from '../Line';
import TypeElement from '../Type';
import IconButtonElement from '../IconButton';
import useMediaQuery from '../useMediaQuery';
import { staticClassName, valueBreakpoints } from '../utils';
const useStyle = styleMethod(theme => ({
  root: {
    margin: 0,
    padding: 0
  },
  expandMore: {
    '&.amaui-IconButton-root': {
      padding: '0',
      width: 'unset',
      height: '100%',
      borderRadius: theme.methods.shape.radius.value(0.25, 'px')
    }
  }
}), {
  name: 'amaui-Breadcrumbs'
});
const Breadcrumbs = /*#__PURE__*/React.forwardRef((props_, ref) => {
  const theme = useAmauiTheme();
  const props = React.useMemo(() => _objectSpread(_objectSpread(_objectSpread({}, theme?.ui?.elements?.all?.props?.default), theme?.ui?.elements?.amauiBreadcrumbs?.props?.default), props_), [props_]);
  const Line = React.useMemo(() => theme?.elements?.Line || LineElement, [theme]);
  const Type = React.useMemo(() => theme?.elements?.Type || TypeElement, [theme]);
  const IconButton = React.useMemo(() => theme?.elements?.IconButton || IconButtonElement, [theme]);
  const {
      max: max_,
      separator = /*#__PURE__*/React.createElement(IconMaterialNavigateNext, {
        size: "small"
      }),
      renderMore,
      IconMore = IconMaterialMoreHoriz,
      Component = 'nav',
      MoreProps,
      className,
      children: children_
    } = props,
    other = _objectWithoutProperties(props, _excluded);
  const {
    classes
  } = useStyle();
  const refs = {
    root: React.useRef(undefined)
  };
  const keys = React.useMemo(() => {
    const result = [];
    const items = [max_];
    items.forEach(item => {
      if (is('object', item)) Object.keys(item).filter(key => theme.breakpoints.media[key]).forEach(key => result.push(key));
    });
    return unique(result);
  }, [max_]);
  const breakpoints = {};
  keys.forEach(key => {
    breakpoints[key] = useMediaQuery(theme.breakpoints.media[key], {
      element: refs.root.current
    });
  });
  let max = valueBreakpoints(max_, undefined, breakpoints, theme);
  max = clamp(max, 2);
  const onExpandMore = () => {
    setChildren(resolve(children_, false));
  };
  const resolve = function () {
    let items_ = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : children_;
    let maxValue = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : max;
    let items = React.Children.toArray(items_).filter(Boolean);
    if (maxValue !== undefined && maxValue >= 2 && maxValue < items.length) {
      const start = [];
      const end = [];
      for (let i = 1; i <= maxValue; i++) {
        // start
        if (i % 2 > 0) start.push(items[Math.floor(i / 2)]);
        // end
        else end.unshift(items[items.length - i / 2]);
      }
      items = [...start, is('function', renderMore) ? renderMore(onExpandMore) : /*#__PURE__*/React.createElement(IconButton, _extends({
        color: "default",
        size: "small",
        onClick: onExpandMore,
        "aria-label": "Expand more path"
      }, MoreProps, {
        className: classNames([staticClassName('Breadcrumbs', theme) && ['amaui-Breadcrumbs-expand-more'], MoreProps?.className, classes.expandMore])
      }), /*#__PURE__*/React.createElement(IconMore, {
        size: "small"
      })), ...end];
    }
    return items;
  };
  const [init, setInit] = React.useState(false);
  const [children, setChildren] = React.useState(resolve);
  React.useEffect(() => {
    setInit(true);
  }, []);
  React.useEffect(() => {
    if (init) setChildren(resolve(children_));
  }, [children_, max]);
  const Separator = is('simple', separator) ? /*#__PURE__*/React.createElement(Type, {
    version: "b2",
    "aria-hidden": true,
    Component: "span"
  }, separator) : /*#__PURE__*/React.cloneElement(separator, {
    'aria-hidden': true
  });
  const childrenToUse = children.flatMap((item, index) => [item, index !== children.length - 1 && Separator].filter(Boolean));
  return /*#__PURE__*/React.createElement(Line, _extends({
    ref: item => {
      if (ref) {
        if (is('function', ref)) ref(item);else ref.current = item;
      }
      refs.root.current = item;
    },
    gap: 1,
    direction: "row",
    align: "center",
    wrap: "wrap",
    "aria-label": "Breadcrumb",
    Component: Component
  }, other, {
    className: classNames([staticClassName('Breadcrumbs', theme) && ['amaui-Breadcrumbs-root'], className, classes.root])
  }), childrenToUse.map((item, index) => {
    const itemProps = {};
    if (index === childrenToUse.length - 1) {
      itemProps['aria-current'] = 'page';
    }
    if (is('simple', item)) {
      return /*#__PURE__*/React.createElement(Type, _extends({
        key: index,
        version: "b2",
        Component: "span"
      }, itemProps, {
        className: classNames([staticClassName('Breadcrumbs', theme) && ['amaui-Breadcrumbs-item'], classes.item])
      }), item);
    }
    return /*#__PURE__*/React.cloneElement(item, _objectSpread({
      key: index,
      className: classNames([staticClassName('Breadcrumbs', theme) && ['amaui-Breadcrumbs-item'], item?.props?.className, classes.item])
    }, itemProps));
  }));
});
Breadcrumbs.displayName = 'amaui-Breadcrumbs';
export default Breadcrumbs;